SET NOCOUNT ON;
go
USE ImEx;
go
--
-- *******************
-- **               **
-- **  S T E P   1  **
-- **               **
-- *******************
--
-- Create a view to aggregate registrations by size
-- * Columns:
--   - Size1 : Number of candidates in the registrations
--   - Avail : Total number of registrations with Size candidates
IF EXISTS
 (SELECT *
  FROM   sys.views
  WHERE  object_id = OBJECT_ID(N'dbo.RegsBySize'))
BEGIN;
  DROP VIEW dbo.RegsBySize;
END;
IF EXISTS
 (SELECT *
  FROM   sys.objects
  WHERE  object_id = OBJECT_ID(N'dbo.RegsBySize')
  AND    type = N'U')
BEGIN;
  DROP TABLE dbo.RegsBySize;
END;
go
CREATE VIEW dbo.RegsBySize
              (Year, Quarter, Size, Avail)
AS SELECT      Year, Quarter, NumCandidates, COUNT(*)
   FROM        dbo.Registrations
   GROUP BY    Year, Quarter, NumCandidates;
go
-- Check intermediate results
-- SELECT * FROM dbo.RegsBySize;
go
--
-- *******************
-- **               **
-- **  S T E P   2  **
-- **               **
-- *******************
--
-- Create the PossibleSessions view
-- * For each allowed session size (1 up to @MaxCandidatesPerSession [reduced to 20 for this version]),
--   this view shows all combinations of registrations that total to that size.
-- * The view is limited to 5 (Size, Cnt) pairs, which suffices for this test data
--   but would be insufficient with the original test data (with session size 100).
-- * To prevent (x,y) // (y,x) "duplicates", we make sure that Size1 is smaller than Size2, etc.
-- * Columns:
--   - SessionSize : Size1 * Cnt1 + ... + Size5 * Cnt5
--   - SizeX       : Number of candidates for Xth pair (X = 1 - 5)
--   - CntX        : Number of registrations used for Xth pair
--   - AvailX      : Total number of registrations with SizeX candidates
--   - RowNum      : Ordinal position when ordering by SessionSize, Size1, Cnt1, ..., Size5, Cnt5
IF EXISTS
 (SELECT *
  FROM   sys.views
  WHERE  object_id = OBJECT_ID(N'dbo.PossibleSessions'))
BEGIN;
  DROP VIEW dbo.PossibleSessions;
END;
IF EXISTS
 (SELECT *
  FROM   sys.objects
  WHERE  object_id = OBJECT_ID(N'dbo.PossibleSessions')
  AND    type = N'U')
BEGIN;
  DROP TABLE dbo.PossibleSessions;
END;
go
CREATE VIEW dbo.PossibleSessions
              (Year,      Quarter,    SessionSize,
               Size1,     Cnt1,       Avail1,
               Size2,     Cnt2,       Avail2,
               Size3,     Cnt3,       Avail3,
               Size4,     Cnt4,       Avail4,
               Size5,     Cnt5,       Avail5,
               RowNum)
AS SELECT      r1.Year,   r1.Quarter, s.Number,
               r1.Size,   n1.Number,  r1.Avail,
               r2.Size,   n2.Number,  r2.Avail,
               r3.Size,   n3.Number,  r3.Avail,
               r4.Size,   n4.Number,  r4.Avail,
               r5.Size,   n5.Number,  r5.Avail,
               ROW_NUMBER() OVER (PARTITION BY r1.Year, r1.Quarter
                                  ORDER BY s.Number,
                                           r1.Size, n1.Number,
                                           r2.Size, n2.Number,
                                           r3.Size, n3.Number,
                                           r4.Size, n4.Number,
                                           r5.Size, n5.Number)
   FROM        dbo.Numbers    AS s                                 -- Session size 1 to max (see WHERE)
   INNER JOIN  dbo.RegsBySize AS r1                                -- Size for first pair:
         ON    r1.Size        <= s.Number                          --  must not exceed session size
   INNER JOIN  dbo.Numbers    AS n1                                -- Cnt for first pair:
         ON    n1.Number      >= 1                                 --  must be at least one,
         AND   n1.Number      <= r1.Avail                          --  at most Avail,
         AND   n1.Number      <= s.Number / r1.Size                --  but not exceed session size
   LEFT  JOIN  dbo.RegsBySize AS r2                                -- Size for second pair:
         ON    r2.Year         = r1.Year                           --  same year,
         AND   r2.Quarter      = r1.Quarter                        --  same quarter,
         AND   r2.Size         > r1.Size                           --  must exceed size of previous pair,
         AND   r2.Size        <= s.Number - (r1.Size * n1.Number)  --  but not REMAINING session size
   LEFT  JOIN  dbo.Numbers    AS n2                                -- Cnt for second pair:
         ON    n2.Number      >= 1                                 --  must be at least one,
         AND   n2.Number      <= r2.Avail                          --  at most Avail,
         AND   n2.Number      <=(s.Number - (r1.Size * n1.Number)) --  but not exceed REMAINING session size
                               / r2.Size
   LEFT  JOIN  dbo.RegsBySize AS r3                                -- Repeat the above for more pairs
         ON    r3.Year        = r2.Year
         AND   r3.Quarter     = r2.Quarter
         AND   r3.Size         > r2.Size
         AND   r3.Size        <= s.Number - (r1.Size * n1.Number
                                           + r2.Size * n2.Number)
   LEFT  JOIN  dbo.Numbers    AS n3
         ON    n3.Number      >= 1
         AND   n3.Number      <= r3.Avail
         AND   n3.Number      <=(s.Number - (r1.Size * n1.Number
                                           + r2.Size * n2.Number)) / r3.Size
   LEFT  JOIN  dbo.RegsBySize AS r4
         ON    r4.Year         = r3.Year
         AND   r4.Quarter      = r3.Quarter
         AND   r4.Size         > r3.Size
         AND   r4.Size        <= s.Number - (r1.Size * n1.Number
                                           + r2.Size * n2.Number
                                           + r3.Size * n3.Number)
   LEFT  JOIN  dbo.Numbers    AS n4
         ON    n4.Number      >= 1
         AND   n4.Number      <= r4.Avail
         AND   n4.Number      <=(s.Number - (r1.Size * n1.Number
                                           + r2.Size * n2.Number
                                           + r3.Size * n3.Number)) / r4.Size
   LEFT  JOIN  dbo.RegsBySize AS r5
         ON    r5.Year         = r4.Year
         AND   r5.Quarter      = r4.Quarter
         AND   r5.Size         > r4.Size
         AND   r5.Size        <= s.Number - (r1.Size * n1.Number
                                           + r2.Size * n2.Number
                                           + r3.Size * n3.Number
                                           + r4.Size * n4.Number)
   LEFT  JOIN  dbo.Numbers    AS n5
         ON    n5.Number      >= 1
         AND   n5.Number      <= r5.Avail
         AND   n5.Number      <=(s.Number - (r1.Size * n1.Number
                                           + r2.Size * n2.Number
                                           + r3.Size * n3.Number
                                           + r4.Size * n4.Number)) / r5.Size
   WHERE       s.Number       >= 1
   AND         s.Number       <= 20                                -- @MaxCandidatesPerSession
   AND         s.Number        = r1.Size * n1.Number               -- All pairs must add up to session size
                               + COALESCE(r2.Size * n2.Number, 0)
                               + COALESCE(r3.Size * n3.Number, 0)
                               + COALESCE(r4.Size * n4.Number, 0)
                               + COALESCE(r5.Size * n5.Number, 0);
go
-- Check intermediate results
-- SELECT * FROM dbo.PossibleSessions;
go
--
-- *******************
-- **               **
-- **  S T E P   3  **
-- **               **
-- *******************
--
-- Create the Solutions view
-- * Combine possible sessions, such that the SUM(SessionSize) = Number of registered candidates,
--   but no registration size is used more often than it is available;
--   these two criteria combine to ensure that all registrations are used exactly once.
-- * The same possible session can be used more than once, if Avail is never exceeded.
-- * The view is limited to 4 possible sessions, which suffices for this test data
--   but would be insufficient with the original test data (with session size 100).
-- * If different possible sessions are used, RowNum is used to prevent (x,y) // (y,x) "duplicates".
-- * Columns:
--   - NumSessions : Number of sessions used
--   - Ranking     : Ordering number, based on number of sessions
--                   + I'm using a trick here that is based on the fact that NULL orders before other values.
--                   + With ROW_NUMBER(), only a single "best" solution will be returned
--                   + With RANK() (or DENSE_RANK()), all "best" solutions will be retured
--   - SizeXY      : Number of candidates for Yth pair (X = 1 - 5) in Xth session (Y = 1 - 4)
--   - CntXY       : Number of registrations used for Yth pair in Xth session
IF EXISTS
 (SELECT *
  FROM   sys.views
  WHERE  object_id = OBJECT_ID(N'dbo.Solutions'))
BEGIN;
  DROP VIEW dbo.Solutions;
END;
IF EXISTS
 (SELECT *
  FROM   sys.objects
  WHERE  object_id = OBJECT_ID(N'dbo.Solutions')
  AND    type = N'U')
BEGIN;
  DROP TABLE dbo.Solutions;
END;
go
CREATE VIEW dbo.Solutions
              (Year, Quarter, NumSessions, Ranking,
               Size11, Cnt11, Size12, Cnt12, Size13, Cnt13, Size14, Cnt14, Size15, Cnt15,
               Size21, Cnt21, Size22, Cnt22, Size23, Cnt23, Size24, Cnt24, Size25, Cnt25,
               Size31, Cnt31, Size32, Cnt32, Size33, Cnt33, Size34, Cnt34, Size35, Cnt35,
               Size41, Cnt41, Size42, Cnt42, Size43, Cnt43, Size44, Cnt44, Size45, Cnt45)
AS WITH
   Quarters   (Year, Quarter, TotalCandidates)
   AS(SELECT   Year, Quarter, SUM(NumCandidates) AS TotalCandidates
      FROM     dbo.Registrations
      GROUP BY Year, Quarter)
   SELECT      q.Year, q.Quarter, CASE WHEN s2.Size1 IS NULL THEN 1
                                       WHEN s3.Size1 IS NULL THEN 2
                                       WHEN s4.Size1 IS NULL THEN 3
                                       ELSE 4
                                  END,
               ROW_NUMBER() OVER (PARTITION BY q.Year, q.Quarter ORDER BY s4.Size1, s3.Size1, s2.Size1),
               s1.Size1, s1.Cnt1, s1.Size2, s1.Cnt2, s1.Size3, s1.Cnt3, s1.Size4, s1.Cnt4, s1.Size5, s1.Cnt5,
               s2.Size1, s2.Cnt1, s2.Size2, s2.Cnt2, s2.Size3, s2.Cnt3, s2.Size4, s2.Cnt4, s2.Size5, s2.Cnt5,
               s3.Size1, s3.Cnt1, s3.Size2, s3.Cnt2, s3.Size3, s3.Cnt3, s3.Size4, s3.Cnt4, s3.Size5, s3.Cnt5,
               s4.Size1, s4.Cnt1, s4.Size2, s4.Cnt2, s4.Size3, s4.Cnt3, s4.Size4, s4.Cnt4, s4.Size5, s4.Cnt5
   FROM        Quarters             AS q
   INNER JOIN  dbo.PossibleSessions AS s1                               -- Use at least one session
         ON    s1.Year               = q.Year
         AND   s1.Quarter            = q.Quarter
   LEFT  JOIN  dbo.PossibleSessions AS s2                               -- Second session:
         ON    s2.Year               = s1.Year                          --  same year,
         AND   s2.Quarter            = s1.Quarter                       --  same quarter,
         AND   s2.RowNum            <= s1.RowNum                        --  (x,y) // (y,x) duplicate prevention,
         AND   s2.SessionSize       <= q.TotalCandidates                --  session size not exceed
                                     - s1.SessionSize                   --    remaining candidates
         AND   s2.Avail1            >= s2.Cnt1                          --  avail1 not exceeded by all sessions,
                                     + CASE  s2.Size1
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END
         AND ( s2.Avail2            IS NULL                             --  avail2 (if any)
            OR s2.Avail2            >= s2.Cnt2                          --         not exceeded by all sessions,
                                     + CASE  s2.Size2
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END)
         AND ( s2.Avail3            IS NULL                             --  avail3 (if any)
            OR s2.Avail3            >= s2.Cnt3                          --         not exceeded by all sessions,
                                     + CASE  s2.Size3
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END)
         AND ( s2.Avail4            IS NULL                             --  avail4 (if any)
            OR s2.Avail4            >= s2.Cnt4                          --         not exceeded by all sessions,
                                     + CASE  s2.Size4
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END)
         AND ( s2.Avail5            IS NULL                             --  avail5 (if any)
            OR s2.Avail5            >= s2.Cnt5                          --         not exceeded by all sessions.
                                     + CASE  s2.Size5
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END)
   LEFT  JOIN  dbo.PossibleSessions AS s3                               -- Third session: same as second
         ON    s3.Year               = s2.Year
         AND   s3.Quarter            = s2.Quarter
         AND   s3.RowNum            <= s2.RowNum
         AND   s3.SessionSize       <= q.TotalCandidates
                                     - s1.SessionSize
                                     - s2.SessionSize
         AND   s3.Avail1            >= s3.Cnt1
                                     + CASE  s3.Size1
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s3.Size1
                                        WHEN s2.Size1 THEN s2.Cnt1
                                        WHEN s2.Size2 THEN s2.Cnt2
                                        WHEN s2.Size3 THEN s2.Cnt3
                                        WHEN s2.Size4 THEN s2.Cnt4
                                        WHEN s2.Size5 THEN s2.Cnt5
                                        ELSE 0
                                       END
         AND ( s3.Avail2            IS NULL
            OR s3.Avail2            >= s3.Cnt2
                                     + CASE  s3.Size2
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s3.Size2
                                        WHEN s2.Size1 THEN s2.Cnt1
                                        WHEN s2.Size2 THEN s2.Cnt2
                                        WHEN s2.Size3 THEN s2.Cnt3
                                        WHEN s2.Size4 THEN s2.Cnt4
                                        WHEN s2.Size5 THEN s2.Cnt5
                                        ELSE 0
                                       END)
         AND ( s3.Avail3            IS NULL
            OR s3.Avail3            >= s3.Cnt3
                                     + CASE  s3.Size3
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s3.Size3
                                        WHEN s2.Size1 THEN s2.Cnt1
                                        WHEN s2.Size2 THEN s2.Cnt2
                                        WHEN s2.Size3 THEN s2.Cnt3
                                        WHEN s2.Size4 THEN s2.Cnt4
                                        WHEN s2.Size5 THEN s2.Cnt5
                                        ELSE 0
                                       END)
         AND ( s3.Avail4            IS NULL
            OR s3.Avail4            >= s3.Cnt4
                                     + CASE  s3.Size4
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s3.Size4
                                        WHEN s2.Size1 THEN s2.Cnt1
                                        WHEN s2.Size2 THEN s2.Cnt2
                                        WHEN s2.Size3 THEN s2.Cnt3
                                        WHEN s2.Size4 THEN s2.Cnt4
                                        WHEN s2.Size5 THEN s2.Cnt5
                                        ELSE 0
                                       END)
         AND ( s3.Avail5            IS NULL
            OR s3.Avail5            >= s3.Cnt5
                                     + CASE  s3.Size5
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s3.Size5
                                        WHEN s2.Size1 THEN s2.Cnt1
                                        WHEN s2.Size2 THEN s2.Cnt2
                                        WHEN s2.Size3 THEN s2.Cnt3
                                        WHEN s2.Size4 THEN s2.Cnt4
                                        WHEN s2.Size5 THEN s2.Cnt5
                                        ELSE 0
                                       END)
   LEFT  JOIN  dbo.PossibleSessions AS s4                               -- Fourth session: same as second and third
         ON    s4.Year               = s3.Year
         AND   s4.Quarter            = s3.Quarter
         AND   s4.RowNum            <= s3.RowNum
         AND   s4.SessionSize       <= q.TotalCandidates
                                     - s1.SessionSize
                                     - s2.SessionSize
                                     - s3.SessionSize
         AND   s4.Avail1            >= s4.Cnt1
                                     + CASE  s4.Size1
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s4.Size1
                                        WHEN s2.Size1 THEN s2.Cnt1
                                        WHEN s2.Size2 THEN s2.Cnt2
                                        WHEN s2.Size3 THEN s2.Cnt3
                                        WHEN s2.Size4 THEN s2.Cnt4
                                        WHEN s2.Size5 THEN s2.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s4.Size1
                                        WHEN s3.Size1 THEN s3.Cnt1
                                        WHEN s3.Size2 THEN s3.Cnt2
                                        WHEN s3.Size3 THEN s3.Cnt3
                                        WHEN s3.Size4 THEN s3.Cnt4
                                        WHEN s3.Size5 THEN s3.Cnt5
                                        ELSE 0
                                       END
         AND ( s4.Avail2            IS NULL
            OR s4.Avail2            >= s4.Cnt2
                                     + CASE  s4.Size2
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s4.Size2
                                        WHEN s2.Size1 THEN s2.Cnt1
                                        WHEN s2.Size2 THEN s2.Cnt2
                                        WHEN s2.Size3 THEN s2.Cnt3
                                        WHEN s2.Size4 THEN s2.Cnt4
                                        WHEN s2.Size5 THEN s2.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s4.Size2
                                        WHEN s3.Size1 THEN s3.Cnt1
                                        WHEN s3.Size2 THEN s3.Cnt2
                                        WHEN s3.Size3 THEN s3.Cnt3
                                        WHEN s3.Size4 THEN s3.Cnt4
                                        WHEN s3.Size5 THEN s3.Cnt5
                                        ELSE 0
                                       END)
         AND ( s4.Avail3            IS NULL
            OR s4.Avail3            >= s4.Cnt3
                                     + CASE  s4.Size3
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s4.Size3
                                        WHEN s2.Size1 THEN s2.Cnt1
                                        WHEN s2.Size2 THEN s2.Cnt2
                                        WHEN s2.Size3 THEN s2.Cnt3
                                        WHEN s2.Size4 THEN s2.Cnt4
                                        WHEN s2.Size5 THEN s2.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s4.Size3
                                        WHEN s3.Size1 THEN s3.Cnt1
                                        WHEN s3.Size2 THEN s3.Cnt2
                                        WHEN s3.Size3 THEN s3.Cnt3
                                        WHEN s3.Size4 THEN s3.Cnt4
                                        WHEN s3.Size5 THEN s3.Cnt5
                                        ELSE 0
                                       END)
         AND ( s4.Avail4            IS NULL
            OR s4.Avail4            >= s4.Cnt4
                                     + CASE  s4.Size4
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s4.Size4
                                        WHEN s2.Size1 THEN s2.Cnt1
                                        WHEN s2.Size2 THEN s2.Cnt2
                                        WHEN s2.Size3 THEN s2.Cnt3
                                        WHEN s2.Size4 THEN s2.Cnt4
                                        WHEN s2.Size5 THEN s2.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s4.Size4
                                        WHEN s3.Size1 THEN s3.Cnt1
                                        WHEN s3.Size2 THEN s3.Cnt2
                                        WHEN s3.Size3 THEN s3.Cnt3
                                        WHEN s3.Size4 THEN s3.Cnt4
                                        WHEN s3.Size5 THEN s3.Cnt5
                                        ELSE 0
                                       END)
         AND ( s4.Avail5            IS NULL
            OR s4.Avail5            >= s4.Cnt5
                                     + CASE  s4.Size5
                                        WHEN s1.Size1 THEN s1.Cnt1
                                        WHEN s1.Size2 THEN s1.Cnt2
                                        WHEN s1.Size3 THEN s1.Cnt3
                                        WHEN s1.Size4 THEN s1.Cnt4
                                        WHEN s1.Size5 THEN s1.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s4.Size5
                                        WHEN s2.Size1 THEN s2.Cnt1
                                        WHEN s2.Size2 THEN s2.Cnt2
                                        WHEN s2.Size3 THEN s2.Cnt3
                                        WHEN s2.Size4 THEN s2.Cnt4
                                        WHEN s2.Size5 THEN s2.Cnt5
                                        ELSE 0
                                       END
                                     + CASE  s4.Size5
                                        WHEN s3.Size1 THEN s3.Cnt1
                                        WHEN s3.Size2 THEN s3.Cnt2
                                        WHEN s3.Size3 THEN s3.Cnt3
                                        WHEN s3.Size4 THEN s3.Cnt4
                                        WHEN s3.Size5 THEN s3.Cnt5
                                        ELSE 0
                                       END)
   WHERE       q.TotalCandidates     = s1.SessionSize
                                     + COALESCE(s2.SessionSize,0)
                                     + COALESCE(s3.SessionSize,0)
                                     + COALESCE(s4.SessionSize,0);
go
-- Check intermediate results
-- SELECT * FROM dbo.Solutions;
go
--
-- *******************
-- **               **
-- **  S T E P   4  **
-- **               **
-- *******************
--
-- Create the BestSolutions view
-- * Same as solutions, but only those with MIN(NumSessions) are included.
-- * Columns:
--   - NumSessions : Number of sessions used
--   - SizeXY      : Number of candidates for Yth pair (X = 1 - 5) in Xth session (Y = 1 - 4)
--   - CntXY       : Number of registrations used for Yth pair in Xth session
IF EXISTS
 (SELECT *
  FROM   sys.views
  WHERE  object_id = OBJECT_ID(N'dbo.BestSolutions'))
BEGIN;
  DROP VIEW dbo.BestSolutions;
END;
IF EXISTS
 (SELECT *
  FROM   sys.objects
  WHERE  object_id = OBJECT_ID(N'dbo.BestSolutions')
  AND    type = N'U')
BEGIN;
  DROP TABLE dbo.BestSolutions;
END;
go
CREATE VIEW dbo.BestSolutions
              (Year, Quarter, NumSessions,
               Size11, Cnt11, Size12, Cnt12, Size13, Cnt13, Size14, Cnt14, Size15, Cnt15,
               Size21, Cnt21, Size22, Cnt22, Size23, Cnt23, Size24, Cnt24, Size25, Cnt25,
               Size31, Cnt31, Size32, Cnt32, Size33, Cnt33, Size34, Cnt34, Size35, Cnt35,
               Size41, Cnt41, Size42, Cnt42, Size43, Cnt43, Size44, Cnt44, Size45, Cnt45)
AS SELECT      Year, Quarter, NumSessions,
               Size11, Cnt11, Size12, Cnt12, Size13, Cnt13, Size14, Cnt14, Size15, Cnt15,
               Size21, Cnt21, Size22, Cnt22, Size23, Cnt23, Size24, Cnt24, Size25, Cnt25,
               Size31, Cnt31, Size32, Cnt32, Size33, Cnt33, Size34, Cnt34, Size35, Cnt35,
               Size41, Cnt41, Size42, Cnt42, Size43, Cnt43, Size44, Cnt44, Size45, Cnt45
   FROM        dbo.Solutions
   WHERE       Ranking = 1;
go
-- Check final results
-- SELECT * FROM dbo.BestSolutions;
go
--
-- *********************
-- **                 **
-- ** Quick and dirty **
-- ** performancetest **
-- **                 **
-- *********************
--
-- Force empty cache
CHECKPOINT;
DBCC FREEPROCCACHE WITH NO_INFOMSGS;
DBCC DROPCLEANBUFFERS WITH NO_INFOMSGS;
go
-- Query final view, keep track of execution time
DECLARE @Start datetime, @End datetime;
SET @Start = CURRENT_TIMESTAMP;

SELECT * FROM dbo.BestSolutions;

SET @End = CURRENT_TIMESTAMP;

-- Return measured execution time
SELECT DATEDIFF(ms, @Start, @End) AS Elapsed;
