SET NOCOUNT ON;
go
USE ImEx;
go
TRUNCATE TABLE dbo.Registrations;

-- Generate some random data for a tiny test set:
-- * Number of quarters unchanged (40);
-- * Maximum registration size reduced to 10;
-- * Number of registations per quarter reduced to 10 (maximum);
-- * Total registrations per quarter may not exceed 70 candidates.
-- The last requirement is to make sure that we'll never need more than 4 sessions
-- of up to 20 candidates (based on the hardcoded maximums in setbased solutions)

DECLARE @NumYears   tinyint  -- Number of years (times 4 quarters)
       ,@MaxSize    smallint -- Maximum registration size
       ,@NumSubj    smallint -- Number of subjects (= rows) in each quarter - max 17500 (for more, change INSERT stmt below)
       ,@MaxCand    smallint -- Maximum number of candidates in a single quarter
SELECT  @NumYears   = 10
       ,@MaxSize    = 10
       ,@NumSubj    = 10
       ,@MaxCand    = 70;

-- Loop counters
DECLARE @y tinyint, @q tinyint, @s smallint;
-- The line below ensures reproducible results - remove it to get truly random results
SET @y = RAND(123); -- Replace 123 with another seed value to get a different set of reproducible results

SET         @y = 1;
WHILE       @y <= @NumYears
BEGIN;
 SET        @q = 1;
 WHILE      @q <= 4
 BEGIN;
   SET      @s = 1;
   WHILE    @s <= @NumSubj
   BEGIN;
    -- Random number generation - same expression for each quarter in this version
    INSERT  dbo.Registrations
           (Year, Quarter,
            SubjCode,
            NumCandidates)
    VALUES (@y + 1994, @q,
            CHAR (@s / 676 + 65) + CHAR ((@s / 26) % 26 + 65) + CHAR (@s % 26 + 65),
            CEILING(@MaxSize * RAND()));
    SET     @s = @s + 1;
  END;
  SET       @q = @q + 1;
 END;
 SET        @y = @y + 1;
END;

-- Check distribution for each quarter before overflow removal
SELECT   Year, Quarter, SUM(NumCandidates) AS TotalCandidates
FROM     dbo.Registrations
GROUP BY Year, Quarter
ORDER BY Year, Quarter;

-- Remove registrations from quarters with too many candidates
WHILE @@ROWCOUNT > 0
BEGIN;
  WITH TheCTE AS
   (SELECT    Year, Quarter,
              SUM(NumCandidates) OVER (PARTITION BY Year, Quarter) AS TotalCandidates,
              ROW_NUMBER() OVER (PARTITION BY Year, Quarter ORDER BY NumCandidates DESC) AS rn
    FROM      dbo.Registrations)
  DELETE FROM TheCTE
  WHERE       TotalCandidates > @MaxCand
  AND         rn = 1                          -- Delete biggest first
END;                                          -- Repeat until deleting stops

-- Check final distribution for each quarter
SELECT   Year, Quarter, SUM(NumCandidates) AS TotalCandidates, COUNT(*) AS NumRegistrations
FROM     dbo.Registrations
GROUP BY Year, Quarter
ORDER BY Year, Quarter;

-- Check average number of sessions per quarter remaining
--  after trimming quarters with too many candidates
SELECT   AVG(cnt)
FROM    (SELECT   1.0 * COUNT(*) AS cnt
         FROM     dbo.Registrations
         GROUP BY Year, Quarter) AS d;
